{-|
   A module for converting Pepa models to Fsp models
-}

module Language.Pepa.Compile.Fsp
   ( pepaModelToFspModel )
where

{- Imported Standard Libraries -}
{- Imported Local Libraries -}
import Language.Pepa.Syntax
   ( QualifiedName      ( .. )
   , ParsedModel        ( .. )
   , ProcessDef
   , ParsedComponent    ( .. )
   , ParsedTrans        ( .. )
   , nameOfAction
   ) 
import Language.Fsp.Syntax
   ( FspModel           ( .. )
   , FspProcessDef
   , FspProcess         ( .. )
   )
import Language.Pepa.Utils
   ( mapSnd )
{- End of Imports -}

{- This doesn't do some modification to the model to get the
   cooperating (and non-cooperating) actions correct, so we still
   have to do that.
-}
pepaModelToFspModel :: ParsedModel -> FspModel
pepaModelToFspModel (ParsedModel _rDefs processDefs mainComp) =
   FspModel $ convertedDefs ++ [ mainDef ]
   where
   mainDef       = convertDef ( Unqualified "System"
                              , mainComp
                              )

   convertedDefs = map convertDef processDefs

   convertDef :: ProcessDef -> FspProcessDef
   convertDef = mapSnd convertProcess

convertProcess :: ParsedComponent -> FspProcess
convertProcess (StopProcess)     =
   FspEnd
convertProcess (IdProcess ident) =
   FspIdProcess ident
convertProcess (PrefixComponent trans next)           =
   FspPrefix (nameOfAction $ pepaTransAction trans) $ convertProcess next
convertProcess (ComponentSum left right)              =
   FspChoice (convertProcess left)
             (convertProcess right)
{-The parallel components -}
convertProcess (Cooperation left _actions right)      =
   FspComposition (convertProcess left)
                  (convertProcess right)
convertProcess (ProcessArray ident size _actions)     =
   FspReplicator ident size
convertProcess (Hiding _ _)                           =
   error "Compile.Fsp: Hiding should have been removed from the model"
convertProcess (CondBehaviour _ _)                    =
   error $ "Compile.Fsp: Conditional behaviour should have been removed "
           ++ "from the model"
