{-|
    This is an example use of the pepa library, it takes in a
    pepa model file and applies some simplifying transformations
    to the model before outputting the same model.
-}

module Main
    ( main )
where

{- Standard library modules imported -}
import System.Environment
     ( getArgs )
import Text.ParserCombinators.Parsec 
    ( ParseError )
{- External Library modules imported -}
{- Local modules imported -}
import Language.Pepa.Syntax
    ( ParsedModel ) 
import Language.Pepa.Parser
    ( parsePepaFile )
import Language.Pepa.Print
    ( hPrintPepaModel )
import Language.Pepa.Transform.Simplify
    ( simplify )
{- End of module imports -}


main :: IO ()
main = do args         <- getArgs
          processArgs args
   
-- Very simplestic options parsing there is no need to go to the
-- length of using 'System.GetOpt' since so far we only have one option.
-- In the future probably the whole library will have a common set of
-- command-line options and we can use that function to parse them.       
processArgs :: [ String ] -> IO () 
processArgs []                          = 
    putStrLn "No input files to analyse"
processArgs ("--print-pepa" : files)    = 
    mapM_ (convertFile [] True)  files
processArgs files                       = 
    mapM_ (convertFile [] False) files

convertFile :: [ String ] -> Bool -> FilePath -> IO ()
convertFile args printPepa file = 
    do contents <- readFile file
       let pModel  = parsePepaFile file contents
           output  = formatResult args printPepa pModel
       putStrLn output

formatResult :: [ String ] -> Bool -> Either ParseError ParsedModel -> String
formatResult _    _     (Left err)    =
    show err
formatResult _args False (Right model) =
    hPrintPepaModel $ simplify [] model
formatResult _args True  (Right model) =
    unlines [ "---- The Original Model ----"
            , hPrintPepaModel model
            , "---- The Simplified Model ----"
            , hPrintPepaModel $ simplify [] model
            ]


